@@ -12,15 +12,18 @@
 // If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.
 //
 
-//! Bitcoin network addresses
+//! Bitcoin network addresses.
 //!
 //! This module defines the structures and functions needed to encode
 //! network addresses in Bitcoin messages.
 //!
 
-use std::{fmt, io, iter};
+use prelude::*;
+
+use core::{fmt, iter};
 use std::net::{SocketAddr, Ipv6Addr, SocketAddrV4, SocketAddrV6, Ipv4Addr, ToSocketAddrs};
 
+use io;
 use network::constants::ServiceFlags;
 use consensus::encode::{self, Decodable, Encodable, VarInt, ReadExt, WriteExt};
 
@@ -35,7 +38,7 @@ pub struct Address {
     pub port: u16
 }
 
-const ONION : [u16; 3] = [0xFD87, 0xD87E, 0xEB43];
+const ONION: [u16; 3] = [0xFD87, 0xD87E, 0xEB43];
 
 impl Address {
     /// Create an address message for a socket
@@ -44,7 +47,7 @@ impl Address {
             SocketAddr::V4(addr) => (addr.ip().to_ipv6_mapped().segments(), addr.port()),
             SocketAddr::V6(addr) => (addr.ip().segments(), addr.port())
         };
-        Address { address: address, port: port, services: services }
+        Address { address, port, services }
     }
 
     /// Extract socket address from an [Address] message.
@@ -55,10 +58,7 @@ impl Address {
         if addr[0..3] == ONION {
             return Err(io::Error::from(io::ErrorKind::AddrNotAvailable));
         }
-        let ipv6 = Ipv6Addr::new(
-            addr[0],addr[1],addr[2],addr[3],
-            addr[4],addr[5],addr[6],addr[7]
-        );
+        let ipv6 = Ipv6Addr::new(addr[0], addr[1], addr[2], addr[3], addr[4], addr[5], addr[6], addr[7]);
         if let Some(ipv4) = ipv6.to_ipv4() {
             Ok(SocketAddr::V4(SocketAddrV4::new(ipv4, self.port)))
         } else {
@@ -68,19 +68,24 @@ impl Address {
 }
 
 fn addr_to_be(addr: [u16; 8]) -> [u16; 8] {
-    [addr[0].to_be(), addr[1].to_be(), addr[2].to_be(), addr[3].to_be(),
-     addr[4].to_be(), addr[5].to_be(), addr[6].to_be(), addr[7].to_be()]
+    // consensus_encode always encodes in LE, and we want to encode in BE.
+    // this utility fn swap bytes before encoding so that the encoded result will be BE
+    let mut result = addr;
+    for word in &mut result {
+        *word = word.swap_bytes();
+    }
+    result
 }
 
 impl Encodable for Address {
     #[inline]
-    fn consensus_encode<S: io::Write>(
-        &self,
-        mut s: S,
-    ) -> Result<usize, io::Error> {
+    fn consensus_encode<S: io::Write>(&self, mut s: S) -> Result<usize, io::Error> {
         let len = self.services.consensus_encode(&mut s)?
             + addr_to_be(self.address).consensus_encode(&mut s)?
-            + self.port.to_be().consensus_encode(s)?;
+
+            // consensus_encode always encodes in LE, and we want to encode in BE.
+            //TODO `len += io::Write::write(&mut e, &self.port.to_be_bytes())?;` when MSRV >= 1.32
+            + self.port.swap_bytes().consensus_encode(s)?;
         Ok(len)
     }
 }
@@ -91,7 +96,7 @@ impl Decodable for Address {
         Ok(Address {
             services: Decodable::consensus_decode(&mut d)?,
             address: addr_to_be(Decodable::consensus_decode(&mut d)?),
-            port: u16::from_be(Decodable::consensus_decode(d)?)
+            port: u16::swap_bytes(Decodable::consensus_decode(d)?)
         })
     }
 }
@@ -101,9 +106,9 @@ impl fmt::Debug for Address {
         let ipv6 = Ipv6Addr::from(self.address);
 
         match ipv6.to_ipv4() {
-            Some(addr) => write!(f, "Address {{services: {}, address: {}, port: {}}}", 
+            Some(addr) => write!(f, "Address {{services: {}, address: {}, port: {}}}",
                 self.services, addr, self.port),
-            None => write!(f, "Address {{services: {}, address: {}, port: {}}}", 
+            None => write!(f, "Address {{services: {}, address: {}, port: {}}}",
                 self.services, ipv6, self.port)
         }
     }
@@ -138,12 +143,11 @@ pub enum AddrV2 {
 impl Encodable for AddrV2 {
     fn consensus_encode<W: io::Write>(&self, e: W) -> Result<usize, io::Error> {
         fn encode_addr<W: io::Write>(mut e: W, network: u8, bytes: &[u8]) -> Result<usize, io::Error> {
-                let len = 
-                    network.consensus_encode(&mut e)? +
-                    VarInt(bytes.len() as u64).consensus_encode(&mut e)? +
-                    bytes.len();
-                e.emit_slice(bytes)?;
-                Ok(len)
+            let len = network.consensus_encode(&mut e)?
+                + VarInt(bytes.len() as u64).consensus_encode(&mut e)?
+                + bytes.len();
+            e.emit_slice(bytes)?;
+            Ok(len)
         }
         Ok(match *self {
             AddrV2::Ipv4(ref addr) => encode_addr(e, 1, &addr.octets())?,
@@ -171,7 +175,7 @@ impl Decodable for AddrV2 {
                 }
                 let addr: [u8; 4] = Decodable::consensus_decode(&mut d)?;
                 AddrV2::Ipv4(Ipv4Addr::new(addr[0], addr[1], addr[2], addr[3]))
-            }, 
+            },
             2 => {
                 if len != 16 {
                     return Err(encode::Error::ParseFailed("Invalid IPv6 address"));
@@ -183,11 +187,8 @@ impl Decodable for AddrV2 {
                 if addr[0..6] == [0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFFFF] {
                     return Err(encode::Error::ParseFailed("IPV4 wrapped address sent with IPv6 network id"));
                 }
-                AddrV2::Ipv6(Ipv6Addr::new(
-                    addr[0],addr[1],addr[2],addr[3],
-                    addr[4],addr[5],addr[6],addr[7]
-                ))
-            }, 
+                AddrV2::Ipv6(Ipv6Addr::new(addr[0], addr[1], addr[2], addr[3], addr[4], addr[5], addr[6], addr[7]))
+            },
             3 => {
                 if len != 10 {
                     return Err(encode::Error::ParseFailed("Invalid TorV2 address"));
@@ -208,7 +209,7 @@ impl Decodable for AddrV2 {
                 }
                 let hash = Decodable::consensus_decode(&mut d)?;
                 AddrV2::I2p(hash)
-            }, 
+            },
             6 => {
                 if len != 16  {
                     return Err(encode::Error::ParseFailed("Invalid CJDNS address"));
@@ -219,17 +220,14 @@ impl Decodable for AddrV2 {
                     return Err(encode::Error::ParseFailed("Invalid CJDNS address"));
                 }
                 let addr = addr_to_be(addr);
-                AddrV2::Cjdns(Ipv6Addr::new(
-                    addr[0],addr[1],addr[2],addr[3],
-                    addr[4],addr[5],addr[6],addr[7]
-                ))
+                AddrV2::Cjdns(Ipv6Addr::new(addr[0], addr[1], addr[2], addr[3], addr[4], addr[5], addr[6], addr[7]))
             },
             _ => {
                 // len already checked above to be <= 512
                 let mut addr = vec![0u8; len as usize];
                 d.read_slice(&mut addr)?;
                 AddrV2::Unknown(network_id, addr)
-            } 
+            }
         })
     }
 }
@@ -255,7 +253,7 @@ impl AddrV2Message {
         match self.addr {
             AddrV2::Ipv4(addr) => Ok(SocketAddr::V4(SocketAddrV4::new(addr, self.port))),
             AddrV2::Ipv6(addr) => Ok(SocketAddr::V6(SocketAddrV6::new(addr, self.port, 0, 0))),
-            _ => return Err(io::Error::from(io::ErrorKind::AddrNotAvailable)),
+            _ => Err(io::Error::from(io::ErrorKind::AddrNotAvailable)),
         }
     }
 }
@@ -266,18 +264,21 @@ impl Encodable for AddrV2Message {
         len += self.time.consensus_encode(&mut e)?;
         len += VarInt(self.services.as_u64()).consensus_encode(&mut e)?;
         len += self.addr.consensus_encode(&mut e)?;
-        len += self.port.to_be().consensus_encode(e)?;
+
+        // consensus_encode always encodes in LE, and we want to encode in BE.
+        //TODO `len += io::Write::write(&mut e, &self.port.to_be_bytes())?;` when MSRV >= 1.32
+        len += self.port.swap_bytes().consensus_encode(e)?;
         Ok(len)
-    }   
+    }
 }
 
 impl Decodable for AddrV2Message {
     fn consensus_decode<D: io::Read>(mut d: D) -> Result<Self, encode::Error> {
-        Ok(AddrV2Message{
+        Ok(AddrV2Message {
             time: Decodable::consensus_decode(&mut d)?,
             services: ServiceFlags::from(VarInt::consensus_decode(&mut d)?.0),
             addr: Decodable::consensus_decode(&mut d)?,
-            port: u16::from_be(Decodable::consensus_decode(d)?),
+            port: u16::swap_bytes(Decodable::consensus_decode(d)?),
         })
     }
 }
@@ -291,7 +292,7 @@ impl ToSocketAddrs for AddrV2Message {
 
 #[cfg(test)]
 mod test {
-    use std::str::FromStr;
+    use core::str::FromStr;
     use super::{AddrV2Message, AddrV2, Address};
     use network::constants::ServiceFlags;
     use std::net::{SocketAddr, IpAddr, Ipv4Addr, Ipv6Addr};
@@ -318,7 +319,7 @@ mod test {
                 services: flags.add(ServiceFlags::WITNESS),
                 address: [0, 0, 0, 0, 0, 0xffff, 0x0a00, 0x0001],
                 port: 8333
-            }), 
+            }),
             "The address is: Address {services: ServiceFlags(NETWORK|WITNESS), address: 10.0.0.1, port: 8333}"
         );
 
@@ -327,7 +328,7 @@ mod test {
                 services: ServiceFlags::NETWORK_LIMITED,
                 address: [0xFD87, 0xD87E, 0xEB43, 0, 0, 0xffff, 0x0a00, 0x0001],
                 port: 8333
-            }), 
+            }),
             "The address is: Address {services: ServiceFlags(NETWORK_LIMITED), address: fd87:d87e:eb43::ffff:a00:1, port: 8333}"
         );
     }
