@@ -18,7 +18,7 @@
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
-//! Merkle Block and Partial Merkle Tree
+//! Merkle Block and Partial Merkle Tree.
 //!
 //! Support proofs that transaction(s) belong to a block.
 //!
@@ -52,8 +52,9 @@
 //! assert_eq!(1, index[0]);
 //! ```
 
-use std::collections::HashSet;
-use std::io;
+use prelude::*;
+
+use io;
 
 use hashes::Hash;
 use hash_types::{Txid, TxMerkleNode};
@@ -188,9 +189,7 @@ impl PartialMerkleTree {
         }
         // there can never be more hashes provided than one for every txid
         if self.hashes.len() as u32 > self.num_transactions {
-            return Err(BadFormat(
-                "Proof contains more hashes than transactions".to_owned(),
-            ));
+            return Err(BadFormat("Proof contains more hashes than transactions".to_owned()));
         };
         // there must be at least one bit per node in the partial tree, and at least one node per hash
         if self.bits.len() < self.hashes.len() {
@@ -245,13 +244,7 @@ impl PartialMerkleTree {
     }
 
     /// Recursive function that traverses tree nodes, storing the data as bits and hashes
-    fn traverse_and_build(
-        &mut self,
-        height: u32,
-        pos: u32,
-        txids: &[Txid],
-        matches: &[bool],
-    ) {
+    fn traverse_and_build(&mut self, height: u32, pos: u32, txids: &[Txid], matches: &[bool]) {
         // Determine whether this node is the parent of at least one matched txid
         let mut parent_of_match = false;
         let mut p = pos << height;
@@ -340,17 +333,14 @@ impl PartialMerkleTree {
     /// Helper method to produce SHA256D(left + right)
     fn parent_hash(left: TxMerkleNode, right: TxMerkleNode) -> TxMerkleNode {
         let mut encoder = TxMerkleNode::engine();
-        left.consensus_encode(&mut encoder).unwrap();
-        right.consensus_encode(&mut encoder).unwrap();
+        left.consensus_encode(&mut encoder).expect("engines don't error");
+        right.consensus_encode(&mut encoder).expect("engines don't error");
         TxMerkleNode::from_engine(encoder)
     }
 }
 
 impl Encodable for PartialMerkleTree {
-    fn consensus_encode<S: io::Write>(
-        &self,
-        mut s: S,
-    ) -> Result<usize, io::Error> {
+    fn consensus_encode<S: io::Write>(&self, mut s: S) -> Result<usize, io::Error> {
         let ret = self.num_transactions.consensus_encode(&mut s)?
             + self.hashes.consensus_encode(&mut s)?;
         let mut bytes: Vec<u8> = vec![0; (self.bits.len() + 7) / 8];
@@ -393,10 +383,10 @@ pub struct MerkleBlock {
 }
 
 impl MerkleBlock {
-    /// Create a MerkleBlock from a block, that should contain proofs for the txids.
+    /// Create a MerkleBlock from a block, that contains proofs for specific txids.
     ///
     /// The `block` is a full block containing the header and transactions and `match_txids` is a
-    /// set containing the transaction ids that should be included in the partial merkle tree.
+    /// function that returns true for the ids that should be included in the partial merkle tree.
     ///
     /// # Examples
     ///
@@ -421,8 +411,8 @@ impl MerkleBlock {
     /// // Create a merkle block containing a single transaction
     /// let txid = Txid::from_hex(
     ///     "5a4ebf66822b0b2d56bd9dc64ece0bc38ee7844a23ff1d7320a88c5fdb2ad3e2").unwrap();
-    /// let match_txids = vec![txid].into_iter().collect();
-    /// let mb = MerkleBlock::from_block(&block, &match_txids);
+    /// let match_txids: Vec<Txid> = vec![txid].into_iter().collect();
+    /// let mb = MerkleBlock::from_block_with_predicate(&block, |t| match_txids.contains(t));
     ///
     /// // Authenticate and extract matched transaction ids
     /// let mut matches: Vec<Txid> = vec![];
@@ -430,32 +420,58 @@ impl MerkleBlock {
     /// assert!(mb.extract_matches(&mut matches, &mut index).is_ok());
     /// assert_eq!(txid, matches[0]);
     /// ```
-    pub fn from_block(block: &Block, match_txids: &HashSet<Txid>) -> Self {
+    pub fn from_block_with_predicate<F>(block: &Block, match_txids: F) -> Self
+    where
+        F: Fn(&Txid) -> bool
+    {
         let block_txids: Vec<_> = block.txdata.iter().map(Transaction::txid).collect();
-        Self::from_header_txids(&block.header, &block_txids, match_txids)
+        Self::from_header_txids_with_predicate(&block.header, &block_txids, match_txids)
     }
 
-    /// Create a MerkleBlock from the block's header and txids, that should contain proofs for match_txids.
+    /// Create a MerkleBlock from a block, that contains proofs for specific txids.
+    #[cfg(feature = "std")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "std")))]
+    #[deprecated(since = "0.26.2", note = "use from_block_with_predicate")]
+    pub fn from_block(block: &Block, match_txids: &::std::collections::HashSet<Txid>) -> Self {
+        Self::from_block_with_predicate(block, |t| match_txids.contains(t))
+    }
+
+    /// Create a MerkleBlock from the block's header and txids, that contain proofs for specific txids.
     ///
     /// The `header` is the block header, `block_txids` is the full list of txids included in the block and
-    /// `match_txids` is a set containing the transaction ids that should be included in the partial merkle tree.
-    pub fn from_header_txids(
+    /// `match_txids` is a function that returns true for the ids that should be included in the partial merkle tree.
+    pub fn from_header_txids_with_predicate<F>(
         header: &BlockHeader,
         block_txids: &[Txid],
-        match_txids: &HashSet<Txid>,
-    ) -> Self {
+        match_txids: F,
+    ) -> Self
+    where
+        F: Fn(&Txid) -> bool
+    {
         let matches: Vec<bool> = block_txids
             .iter()
-            .map(|txid| match_txids.contains(txid))
+            .map(match_txids)
             .collect();
 
-        let pmt = PartialMerkleTree::from_txids(&block_txids, &matches);
+        let pmt = PartialMerkleTree::from_txids(block_txids, &matches);
         MerkleBlock {
             header: *header,
             txn: pmt,
         }
     }
 
+    /// Create a MerkleBlock from the block's header and txids, that should contain proofs for match_txids.
+    #[cfg(feature = "std")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "std")))]
+    #[deprecated(since = "0.26.2", note = "use from_header_txids_with_predicate")]
+    pub fn from_header_txids(
+        header: &BlockHeader,
+        block_txids: &[Txid],
+        match_txids: &::std::collections::HashSet<Txid>,
+    ) -> Self {
+        Self::from_header_txids_with_predicate(header, block_txids, |t| match_txids.contains(t))
+    }
+
     /// Extract the matching txid's represented by this partial merkle tree
     /// and their respective indices within the partial tree.
     /// returns Ok(()) on success, or error in case of failure
@@ -475,10 +491,7 @@ impl MerkleBlock {
 }
 
 impl Encodable for MerkleBlock {
-    fn consensus_encode<S: io::Write>(
-        &self,
-        mut s: S,
-    ) -> Result<usize, io::Error> {
+    fn consensus_encode<S: io::Write>(&self, mut s: S) -> Result<usize, io::Error> {
         let len = self.header.consensus_encode(&mut s)?
             + self.txn.consensus_encode(s)?;
         Ok(len)
@@ -496,7 +509,7 @@ impl Decodable for MerkleBlock {
 
 #[cfg(test)]
 mod tests {
-    use std::cmp::min;
+    use core::cmp::min;
 
     use hashes::Hash;
     use hashes::hex::{FromHex, ToHex};
@@ -521,7 +534,7 @@ mod tests {
 
             // Calculate the merkle root and height
             let hashes = txids.iter().map(|t| t.as_hash());
-            let merkle_root_1: TxMerkleNode = bitcoin_merkle_root(hashes).into();
+            let merkle_root_1: TxMerkleNode = bitcoin_merkle_root(hashes).expect("hashes is not empty").into();
             let mut height = 1;
             let mut ntx = num_tx;
             while ntx > 1 {
@@ -643,9 +656,9 @@ mod tests {
 
         let txid1 = txids[0];
         let txid2 = txids[1];
-        let txids = txids.into_iter().collect();
+        let txids = vec![txid1, txid2];
 
-        let merkle_block = MerkleBlock::from_block(&block, &txids);
+        let merkle_block = MerkleBlock::from_block_with_predicate(&block, |t| txids.contains(t));
 
         assert_eq!(merkle_block.header.block_hash(), block.block_hash());
 
@@ -673,12 +686,12 @@ mod tests {
     #[test]
     fn merkleblock_construct_from_txids_not_found() {
         let block = get_block_13b8a();
-        let txids = ["c0ffee00003bafa802c8aa084379aa98d9fcd632ddc2ed9782b586ec87451f20"]
+        let txids: Vec<Txid> = ["c0ffee00003bafa802c8aa084379aa98d9fcd632ddc2ed9782b586ec87451f20"]
             .iter()
             .map(|hex| Txid::from_hex(hex).unwrap())
             .collect();
 
-        let merkle_block = MerkleBlock::from_block(&block, &txids);
+        let merkle_block = MerkleBlock::from_block_with_predicate(&block, |t| txids.contains(t));
 
         assert_eq!(merkle_block.header.block_hash(), block.block_hash());
 
